"use strict";

//========================================
// リスト6 公開鍵をバイナリ形式へ変換する
// （serverKeyはserverkey.jsで読み込んでいる）
let serverKeyBin = urlsafeB64toBin(serverKey);

function urlsafeB64toBin(urlsafeb64){
  // 通常のbase64になるように文字を置換
  let b64 = urlsafeb64.replace(/-/g, '+')
                      .replace(/_/g, '/');
  let raw = window.atob(b64);
  // 1文字ずつUint8Arrayに詰めていく
  let result = new Uint8Array(raw.length);
  for(let i=0; i<raw.length; i++){
    result[i] = raw.charCodeAt(i);
  }
  return result;
}
// リスト6 公開鍵をバイナリ形式へ変換する ここまで
//========================================

/**
 * Service Worker の登録処理
 */
document.querySelector('#subscribe').addEventListener('click', registration, false);
function registration(){
  //========================================
  // リスト4 Service Workerの登録
  // リスト7 プッシュ通知購読
  // (1)ServiceWorker登録要求
  navigator.serviceWorker.register('/webpush/sw.js',
                                   {scope: '/webpush/'})
    .then((result) => {
      // (3)登録完了
      console.log('Service Worker を登録しました');
      return navigator.serviceWorker.ready;
    })
    .catch(() => {
      console.log('Service Worker の登録に失敗しました');
    })
    .then((result) => {
      // (4)通知購読要求
      return result.pushManager.subscribe({
        userVisibleOnly: true,
        applicationServerKey: serverKeyBin
      });
    })
    .then((subscription) => {
      // (5)エンドポイントなどの取得
      let subscriptionInfo = JSON.stringify(subscription);
      // => {
      //   "endpoint": "<エンドポイントURI>",
      //   "keys": {
      //     "auth": "<urlsafe base64エンコード済共有鍵>",
      //     "p256dh": "<  〃  公開鍵>"
      //   }
      // }
      // (6)エンドポイントなどをアプリケーションサーバに送信
      // fetch(
      //   '/api/push/subscribe',
      //   {
      //     method: 'POST',
      //     headers: new Headers({
      //       'Content-Type': 'application/json'
      //     }),
      //     body: subscription
      //   }
      // );
      // 実際のアプリケーションであれば
      // エンドポイントなどをアプリケーションサーバに送信するのだが、
      // このサンプルではサーバには送信せず画面に出力し、
      // コントロールパネルに入力してもらう
      document.querySelector('#subscriptionInfo').value = subscriptionInfo;
      console.log('プッシュ通知を購読しました');
    })
    .catch((e) => {
      console.log(e);
      console.log('プッシュ通知の購読に失敗しました');
    });
  // リスト4 Service Workerの登録
  // リスト7 プッシュ通知購読 ここまで
  //========================================
}
